<?php

namespace Drupal\mark_as_read\Controller;

use Drupal\Component\Utility\Xss;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Url;
use Drupal\mark_as_read\Entity\RegulationInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class RegulationController.
 *
 *  Returns responses for Regulation routes.
 */
class RegulationController extends ControllerBase implements ContainerInjectionInterface {

  /**
   * The date formatter.
   *
   * @var \Drupal\Core\Datetime\DateFormatter
   */
  protected $dateFormatter;

  /**
   * The renderer.
   *
   * @var \Drupal\Core\Render\Renderer
   */
  protected $renderer;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    $instance = parent::create($container);
    $instance->dateFormatter = $container->get('date.formatter');
    $instance->renderer = $container->get('renderer');
    return $instance;
  }

  /**
   * Displays a Regulation revision.
   *
   * @param int $regulation_revision
   *   The Regulation revision ID.
   *
   * @return array
   *   An array suitable for drupal_render().
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function revisionShow($regulation_revision) {
    $regulation = $this->entityTypeManager()->getStorage('regulation')
      ->loadRevision($regulation_revision);
    $view_builder = $this->entityTypeManager()->getViewBuilder('regulation');

    return $view_builder->view($regulation);
  }

  /**
   * Page title callback for a Regulation revision.
   *
   * @param int $regulation_revision
   *   The Regulation revision ID.
   *
   * @return string
   *   The page title.
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function revisionPageTitle($regulation_revision) {
    $regulation = $this->entityTypeManager()->getStorage('regulation')
      ->loadRevision($regulation_revision);
    return $this->t('Revision of %title from %date', [
      '%title' => $regulation->label(),
      '%date' => $this->dateFormatter->format($regulation->getRevisionCreationTime()),
    ]);
  }

  /**
   * Generates an overview table of older revisions of a Regulation.
   *
   * @param \Drupal\mark_as_read\Entity\RegulationInterface $regulation
   *   A Regulation object.
   *
   * @return array
   *   An array as expected by drupal_render().
   *
   * @throws \Drupal\Component\Plugin\Exception\InvalidPluginDefinitionException
   * @throws \Drupal\Component\Plugin\Exception\PluginNotFoundException
   */
  public function revisionOverview(RegulationInterface $regulation) {
    $account = $this->currentUser();
    $regulation_storage = $this->entityTypeManager()->getStorage('regulation');

    $langcode = $regulation->language()->getId();
    $langname = $regulation->language()->getName();
    $languages = $regulation->getTranslationLanguages();
    $has_translations = (count($languages) > 1);
    $build['#title'] = $has_translations ? $this->t('@langname revisions for %title', ['@langname' => $langname, '%title' => $regulation->label()]) : $this->t('Revisions for %title', ['%title' => $regulation->label()]);

    $header = [$this->t('Revision'), $this->t('Operations')];
    $revert_permission = (($account->hasPermission("revert all regulation revisions") || $account->hasPermission('administer regulation entities')));
    $delete_permission = (($account->hasPermission("delete all regulation revisions") || $account->hasPermission('administer regulation entities')));

    $rows = [];

    $vids = $regulation_storage->revisionIds($regulation);

    $latest_revision = TRUE;

    foreach (array_reverse($vids) as $vid) {
      /** @var \Drupal\mark_as_read\Entity\RegulationInterface $revision */
      $revision = $regulation_storage->loadRevision($vid);
      // Only show revisions that are affected by the language that is being
      // displayed.
      if ($revision->hasTranslation($langcode) && $revision->getTranslation($langcode)->isRevisionTranslationAffected()) {
        $username = [
          '#theme' => 'username',
          '#account' => $revision->getRevisionUser(),
        ];

        // Use revision link to link to revisions that are not active.
        $date = $this->dateFormatter->format($revision->getRevisionCreationTime(), 'short');
        if ($vid != $regulation->getRevisionId()) {
          $link = $this->l($date, new Url('entity.regulation.revision', [
            'regulation' => $regulation->id(),
            'regulation_revision' => $vid,
          ]));
        }
        else {
          $link = $regulation->link($date);
        }

        $row = [];
        $column = [
          'data' => [
            '#type' => 'inline_template',
            '#template' => '{% trans %}{{ date }} by {{ username }}{% endtrans %}{% if message %}<p class="revision-log">{{ message }}</p>{% endif %}',
            '#context' => [
              'date' => $link,
              'username' => $this->renderer->renderPlain($username),
              'message' => [
                '#markup' => $revision->getRevisionLogMessage(),
                '#allowed_tags' => Xss::getHtmlTagList(),
              ],
            ],
          ],
        ];
        $row[] = $column;

        if ($latest_revision) {
          $row[] = [
            'data' => [
              '#prefix' => '<em>',
              '#markup' => $this->t('Current revision'),
              '#suffix' => '</em>',
            ],
          ];
          foreach ($row as &$current) {
            $current['class'] = ['revision-current'];
          }
          $latest_revision = FALSE;
        }
        else {
          $links = [];
          if ($revert_permission) {
            $links['revert'] = [
              'title' => $this->t('Revert'),
              'url' => $has_translations ?
              Url::fromRoute('entity.regulation.translation_revert', [
                'regulation' => $regulation->id(),
                'regulation_revision' => $vid,
                'langcode' => $langcode,
              ]) :
              Url::fromRoute('entity.regulation.revision_revert', [
                'regulation' => $regulation->id(),
                'regulation_revision' => $vid,
              ]),
            ];
          }

          if ($delete_permission) {
            $links['delete'] = [
              'title' => $this->t('Delete'),
              'url' => Url::fromRoute('entity.regulation.revision_delete', [
                'regulation' => $regulation->id(),
                'regulation_revision' => $vid,
              ]),
            ];
          }

          $row[] = [
            'data' => [
              '#type' => 'operations',
              '#links' => $links,
            ],
          ];
        }

        $rows[] = $row;
      }
    }

    $build['regulation_revisions_table'] = [
      '#theme' => 'table',
      '#rows' => $rows,
      '#header' => $header,
    ];

    return $build;
  }

}
